//+------------------------------------------------------------------+
//|                                    Traders Dynamic Index.mq4     |
//|                                    Copyright  2006, Dean Malone |
//+------------------------------------------------------------------+

//+------------------------------------------------------------------+
//|                                                                  |
//|                     Traders Dynamic Index                        |
//|                                                                  |
//|  This hybrid indicator is developed to assist traders in their   |
//|  ability to decipher and monitor market conditions related to    |
//|  trend direction, market strength, and market volatility.        |
//|                                                                  | 
//|  Even though comprehensive, the T.D.I. is easy to read and use.  |
//|                                                                  |
//|  Green line = RSI Price line                                     |
//|  Red line = Trade Signal line                                    |
//|  Blue lines = Volatility Band                                    | 
//|  Yellow line = Market Base Line                                  |  
//|                                                                  |
//|  Trend Direction - Immediate and Overall                         |
//|   Immediate = Green over Red...price action is moving up.        |
//|               Red over Green...price action is moving down.      |
//|                                                                  |   
//|   Overall = Yellow line trends up and down generally between the |
//|             lines 32 & 68. Watch for Yellow line to bounces off  |
//|             these lines for market reversal. Trade long when     |
//|             price is above the Yellow line, and trade short when |
//|             price is below.                                      |        
//|                                                                  |
//|  Market Strength & Volatility - Immediate and Overall            |
//|   Immediate = Green Line - Strong = Steep slope up or down.      | 
//|                            Weak = Moderate to Flat slope.        |
//|                                                                  |               
//|   Overall = Blue Lines - When expanding, market is strong and    |
//|             trending. When constricting, market is weak and      |
//|             in a range. When the Blue lines are extremely tight  |                                                       
//|             in a narrow range, expect an economic announcement   | 
//|             or other market condition to spike the market.       |
//|                                                                  |               
//|                                                                  |
//|  Entry conditions                                                |
//|   Scalping  - Long = Green over Red, Short = Red over Green      |
//|   Active - Long = Green over Red & Yellow lines                  |
//|            Short = Red over Green & Yellow lines                 |    
//|   Moderate - Long = Green over Red, Yellow, & 50 lines           |
//|              Short= Red over Green, Green below Yellow & 50 line |
//|                                                                  |
//|  Exit conditions*                                                |   
//|   Long = Green crosses below Red                                 |
//|   Short = Green crosses above Red                                |
//|   * If Green crosses either Blue lines, consider exiting when    |
//|     when the Green line crosses back over the Blue line.         |
//|                                                                  |
//|                                                                  |
//|  IMPORTANT: The default settings are well tested and proven.     |
//|             But, you can change the settings to fit your         |
//|             trading style.                                       |
//|                                                                  |
//|                                                                  |
//|  Price & Line Type settings:                           |                
//|   RSI Price settings                                             |               
//|   0 = Close price     [DEFAULT]                                  |               
//|   1 = Open price.                                                |               
//|   2 = High price.                                                |               
//|   3 = Low price.                                                 |               
//|   4 = Median price, (high+low)/2.                                |               
//|   5 = Typical price, (high+low+close)/3.                         |               
//|   6 = Weighted close price, (high+low+close+close)/4.            |               
//|                                                                  |               
//|   RSI Price Line & Signal Line Type settings                                   |               
//|   0 = Simple moving average       [DEFAULT]                      |               
//|   1 = Exponential moving average                                 |               
//|   2 = Smoothed moving average                                    |               
//|   3 = Linear weighted moving average                             |               
//|                                                                  |
//|   Good trading,                                                  |   
//|                                                                  |
//|   Dean                                                           |                              
//+------------------------------------------------------------------+

#property indicator_separate_window

#property indicator_buffers 6
#property indicator_color1 clrBlack
#property indicator_color2 clrDodgerBlue
#property indicator_color3 clrYellow
#property indicator_color4 clrDodgerBlue
#property indicator_color5 clrGreen
#property indicator_color6 clrRed

extern int RSI_Period = 10;


double RSIBuf[],UpZone[],MdZone[],DnZone[],MaBuf[],MbBuf[];

int init() {

   IndicatorShortName("TDI_RSI_10");
   SetIndexBuffer(0, RSIBuf);
   SetIndexBuffer(1, UpZone);
   SetIndexBuffer(2, MdZone);
   SetIndexBuffer(3, DnZone);
   SetIndexBuffer(4, MaBuf);
   SetIndexBuffer(5, MbBuf);
   SetIndexStyle(0,DRAW_NONE); 
   SetIndexStyle(1,DRAW_LINE); 
   SetIndexStyle(2,DRAW_LINE,2,1);
   SetIndexStyle(3,DRAW_LINE);
   SetIndexStyle(4,DRAW_LINE,0,2);
   SetIndexStyle(5,DRAW_LINE,0,1);     
   SetIndexLabel(0, NULL);
   SetIndexLabel(1, "UpZone");
   SetIndexLabel(2, "MBL");
   SetIndexLabel(3, "MidZone");
   SetIndexLabel(4, "TDI");
   SetIndexLabel(5, "TDI_Signal");
   SetLevelValue(0,75);
   SetLevelValue(1,68);
   SetLevelValue(2,50);   
   SetLevelValue(3,32);
   SetLevelValue(4,25);
   SetLevelStyle(STYLE_DOT,1,DarkSlateGray);
   
   return (0);
}

int deinit() {
   return (0);
}

int start() {

double MA,RSI[];  
   int counted_bars = IndicatorCounted();
   if (counted_bars < 0) return (-1);
   if (counted_bars > 0) counted_bars--;
   int limit = MathMin(Bars - counted_bars, Bars - 1);
   
   ArrayResize(RSI, 34);
   
   for (int i = limit; i >= 0; i--) { // the fixed http://www.forexfactory.com/showthread.php?p=4621048#post4621048
      RSIBuf[i] = iRSI(NULL, 0, RSI_Period, PRICE_TYPICAL, i);
      MA = 0;
      for (int x = i; x < i + 34; x++) {
         RSI[x - i] = RSIBuf[x];
         MA += RSIBuf[x] / 34.0;
      }
      UpZone[i] = MA + 1.6185 * StdDev(RSI, 34);
      DnZone[i] = MA - 1.6185 * StdDev(RSI, 34);
      MdZone[i] = (UpZone[i] + DnZone[i]) / 2.0;
   }
   for (i = limit - 1; i >= 0; i--) {
      MaBuf[i] = iMAOnArray(RSIBuf, 0, 2, 0, MODE_SMMA, i);
      MbBuf[i] = iMAOnArray(RSIBuf, 0, 7, 0, MODE_SMA, i);


   }
   return (0);
}

double StdDev(double &Data[], int Per) {
   return (MathSqrt(Variance(Data, Per)));
}

double Variance(double &Data[], int Per) {
   double sum;
   double ssum;
   for (int i = 0; i < Per; i++) {
      sum += Data[i];
      ssum += MathPow(Data[i], 2);
   }
   return ((ssum * Per - sum * sum) / (Per * (Per - 1)));
}