/*------------------------------------------------------------------------------------
   Name: TMS-TDI-RT-Replica.mq4
   
   Description: A clone of the TDI indicator. 
                The volatility bands and the market base line are not exactly the same 
                but they are close enough. 
                	          
-------------------------------------------------------------------------------------*/
// Indicator properties
#property copyright "v2vboni"
#property link      "http://www.forexfactory.com/showthread.php?t=518993"
#property strict
#property version     "1.0"
#property description "A Replica of the TDI-RT-Clone indicator by xaphod"
#property description "Use with TMS_Angleator_Green.mq4 by Griffinn Soul"
#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 clrMediumBlue
#property indicator_width1 1
#property indicator_color2 clrMediumBlue
#property indicator_width2 1
#property indicator_color3 clrYellow
#property indicator_width3 1
#property indicator_color4 clrRed
#property indicator_width4 2
#property indicator_color5 clrGreen
#property indicator_width5 2
#property indicator_color6 CLR_NONE
#property indicator_width6 1
#property indicator_level1 32
#property indicator_level2 50
#property indicator_level3 68
#property indicator_levelstyle STYLE_DOT
#property indicator_levelcolor DimGray

#define INDICATOR_NAME "TMS-TDI-RT-Replica"

// Indicator parameters
extern int                RSI_Period=13;            
extern ENUM_APPLIED_PRICE RSI_Price=PRICE_CLOSE; 
extern int                Volatility_Band=34;
extern int                RSISignal_Period=2;   
extern ENUM_MA_METHOD     RSISignal_Mode=MODE_SMA;
extern int                TradeSignal_Period=7;      
extern ENUM_MA_METHOD     TradeSignal_Mode=MODE_SMA; 

extern int    VolatilityBandMAMode     = 0;
extern double VolatilityBandMultiplier = 1.6185;
extern int    T3Period                 = 1;
extern int    T3Price                  = PRICE_CLOSE;
extern double T3Hot                    = 1.0;
extern bool   T3Original               = false;


double workT3[][6];
double workT3Coeffs[][6];
#define _period 0
#define _c1     1
#define _c2     2
#define _c3     3
#define _c4     4
#define _alpha  5

double workRsi[][3];
#define _price  0
#define _change 1
#define _changa 2

// Global module varables
double gdaRSI[];
double gdaRSISig[];
double gdaTradeSig[];
double gdaMktBase[];
double gdaVolaTop[];
double gdaVolaBtm[];


//-----------------------------------------------------------------------------
// function: init()
// Description: Custom indicator initialization function.
//-----------------------------------------------------------------------------
int init() {
  SetIndexStyle(0, DRAW_LINE);
  SetIndexBuffer(0, gdaVolaTop);
  SetIndexLabel(0,"Volatility Top");   
  SetIndexStyle(1, DRAW_LINE);
  SetIndexBuffer(1, gdaVolaBtm);
  SetIndexLabel(1,"Volatility Bottom");   
  SetIndexStyle(2, DRAW_LINE);
  SetIndexBuffer(2, gdaMktBase);
  SetIndexLabel(3,"Market Base");
  SetIndexStyle(3, DRAW_LINE);
  SetIndexBuffer(3, gdaTradeSig);
  SetIndexLabel(3,"Trade Signal");
  SetIndexStyle(4, DRAW_LINE);
  SetIndexBuffer(4, gdaRSISig);
  SetIndexLabel(4,"RSI Signal");    
  SetIndexStyle(5, DRAW_NONE);
  SetIndexBuffer(5, gdaRSI);
  SetIndexLabel(5,NULL);
  IndicatorDigits(1);
  IndicatorShortName(INDICATOR_NAME);
  return(0);
}

//-----------------------------------------------------------------------------
// function: deinit()
// Description: Custom indicator deinitialization function.
//-----------------------------------------------------------------------------
int deinit() {
   return (0);
}


///-----------------------------------------------------------------------------
// function: start()
// Description: Custom indicator iteration function.
//-----------------------------------------------------------------------------
int start() {
  int iNewBars, iCountedBars, i;  
  
  // Get unprocessed bars
  iCountedBars=IndicatorCounted();
  if(iCountedBars < 0) return (-1); 
  if(iCountedBars>0) iCountedBars--;
  iNewBars=MathMin(Bars-iCountedBars, Bars-1);
  // Calc TDI data
  for(i=iNewBars-1; i>=0; i--) {
    gdaRSI[i] = iT3(iRsi(iMA(NULL,0,1,0,MODE_SMA,RSI_Price,i),RSI_Period,i,0),T3Period,T3Hot,T3Original,i);
  }
  for(i=iNewBars-1; i>=0; i--) {  
    gdaRSISig[i]=iMAOnArray(gdaRSI,0,RSISignal_Period,0,RSISignal_Mode,i); 
    gdaTradeSig[i]=iMAOnArray(gdaRSI,0,TradeSignal_Period,0,TradeSignal_Mode,i);
    
    double deviation = iStdDevOnArray(gdaRSI,0,Volatility_Band,0,VolatilityBandMAMode,i);
    double average   = iMAOnArray(gdaRSI,0,Volatility_Band,0,VolatilityBandMAMode,i);

    gdaVolaTop[i]=average+VolatilityBandMultiplier*deviation;
    gdaVolaBtm[i]=average-VolatilityBandMultiplier*deviation;  
    gdaMktBase[i]=average; //middle line
  } 
  return(0);
}
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
double iT3(double price, double period, double hot, bool original, int i, int forInstance=0)   {
//+------------------------------------------------------------------+
  if (ArrayRange(workT3,0) !=Bars)                  ArrayResize(workT3,Bars);
  if (ArrayRange(workT3Coeffs,0) < (forInstance+1)) ArrayResize(workT3Coeffs,forInstance+1);

  if (workT3Coeffs[forInstance][_period] != period)   {
    workT3Coeffs[forInstance][_period] = period;
    double a = hot;
    workT3Coeffs[forInstance][_c1] = -a*a*a;
    workT3Coeffs[forInstance][_c2] = 3*a*a+3*a*a*a;
    workT3Coeffs[forInstance][_c3] = -6*a*a-3*a-3*a*a*a;
    workT3Coeffs[forInstance][_c4] = 1+3*a+a*a*a+3*a*a;
    if (original)
      workT3Coeffs[forInstance][_alpha] = 2.0/(1.0 + period);
    else 
      workT3Coeffs[forInstance][_alpha] = 2.0/(2.0 + (period-1.0)/2.0);
  }
  int buffer = forInstance*6;
  int r = Bars-i-1;
  if (r == 0)   {
    workT3[r][0+buffer] = price;
    workT3[r][1+buffer] = price;
    workT3[r][2+buffer] = price;
    workT3[r][3+buffer] = price;
    workT3[r][4+buffer] = price;
    workT3[r][5+buffer] = price;
  } else {
    workT3[r][0+buffer] = workT3[r-1][0+buffer]+workT3Coeffs[forInstance][_alpha]*(price              -workT3[r-1][0+buffer]);
    workT3[r][1+buffer] = workT3[r-1][1+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][0+buffer]-workT3[r-1][1+buffer]);
    workT3[r][2+buffer] = workT3[r-1][2+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][1+buffer]-workT3[r-1][2+buffer]);
    workT3[r][3+buffer] = workT3[r-1][3+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][2+buffer]-workT3[r-1][3+buffer]);
    workT3[r][4+buffer] = workT3[r-1][4+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][3+buffer]-workT3[r-1][4+buffer]);
    workT3[r][5+buffer] = workT3[r-1][5+buffer]+workT3Coeffs[forInstance][_alpha]*(workT3[r][4+buffer]-workT3[r-1][5+buffer]);
  }
  return(workT3Coeffs[forInstance][_c1]*workT3[r][5+buffer] + 
         workT3Coeffs[forInstance][_c2]*workT3[r][4+buffer] + 
         workT3Coeffs[forInstance][_c3]*workT3[r][3+buffer] + 
         workT3Coeffs[forInstance][_c4]*workT3[r][2+buffer]);
}


//+------------------------------------------------------------------+
double iRsi(double price, double period, int shift, int forz=0)   {
//+------------------------------------------------------------------+
  if (ArrayRange(workRsi,0)!=Bars) 
    ArrayResize(workRsi,Bars);
  int    z     = forz*3; 
  int    i     = Bars-shift-1;
  double alpha = 1.0/(double)period; 
  workRsi[i][_price+z] = price;
  if (i<period)      {
    int k; 
    double sum = 0; 
    for (k=0; k<period && (i-k-1)>=0; k++) 
      sum += MathAbs(workRsi[i-k][_price+z]-workRsi[i-k-1][_price+z]);
    workRsi[i][_change+z] = (workRsi[i][_price+z]-workRsi[0][_price+z])/MathMax(k,1);
    workRsi[i][_changa+z] = sum/MathMax(k,1);
  } else {
    double change = workRsi[i][_price+z]-workRsi[i-1][_price+z];
    workRsi[i][_change+z] = workRsi[i-1][_change+z] + alpha*(        change  - workRsi[i-1][_change+z]);
    workRsi[i][_changa+z] = workRsi[i-1][_changa+z] + alpha*(MathAbs(change) - workRsi[i-1][_changa+z]);
  }
  if (workRsi[i][_changa+z] != 0)
    return(50.0*(workRsi[i][_change+z]/workRsi[i][_changa+z]+1));
  else  
    return(0);
}
